"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileTabs = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("@liqvid/utils/react");
const codemirror_1 = require("@lqv/codemirror");
const classnames_1 = __importDefault(require("classnames"));
const react_2 = require("react");
const zustand_1 = require("zustand");
const store_1 = require("../store.cjs");
const utils_1 = require("../utils.cjs");
const selector = (state) => [state.activeGroup, state.groups[state.activeGroup]?.activeFile];
/**
 * File selector component.
 */
function FileTabs({ className }) {
    const store = (0, store_1.useBoothStore)();
    const [activeGroup, activeFilename] = (0, zustand_1.useStore)(store, selector);
    const group = store.getState().groups[activeGroup];
    const { recorder } = store.getState();
    const select = (0, react_2.useCallback)((filename) => {
        // record event
        // @ts-expect-error TODO fix this
        if (recorder?.manager?.active) {
            recorder.capture(undefined, codemirror_1.selectCmd + filename);
        }
        // set state
        store.setState((state) => ({
            groups: {
                ...state.groups,
                [state.activeGroup]: {
                    ...state.groups[state.activeGroup],
                    activeFile: filename,
                },
            },
        }));
        // focus editor
        const state = store.getState();
        const view = state.groups[state.activeGroup]?.files.find((_) => _.filename === filename)?.view;
        if (view) {
            // XXX yikes
            setTimeout(() => view.focus());
        }
    }, [recorder]);
    const events = (0, react_2.useMemo)(() => (0, react_1.onClick)((e) => {
        select(e.currentTarget.textContent.trim());
    }), [select]);
    // set class
    (0, react_2.useEffect)(() => {
        // keyboard shortcuts
        const selectShortcuts = {};
        for (let i = 1; i <= 9; ++i) {
            selectShortcuts[`Mod-${i}`] = {
                key: `Mod-${i}`,
                run: () => {
                    const state = store.getState();
                    const group = state.groups[state.activeGroup];
                    if (group.files.length < i)
                        return false;
                    select(group.files[i - 1].filename);
                    return true;
                },
            };
        }
        // set class
        store.setState((prev) => ({
            // set class
            classNames: prev.classNames.concat("multifile"),
            // shortcuts
            shortcuts: {
                ...prev.shortcuts,
                ...selectShortcuts,
            },
        }));
    }, []);
    if (!group)
        return null;
    return ((0, jsx_runtime_1.jsx)("div", { className: (0, classnames_1.default)("lqv-file-tabs", className), role: "tablist", children: group.files.map(({ filename }) => ((0, jsx_runtime_1.jsx)("button", { "aria-controls": utils_1.ids.editorPanel({ filename, group: activeGroup }), "aria-selected": activeFilename === filename, className: `lqv-filetype-${getFileType(filename)}`, id: utils_1.ids.fileTab({ filename, group: activeGroup }), role: "tab", ...events, children: filename }, filename))) }));
}
exports.FileTabs = FileTabs;
/**
 * Get file extension.
 * @param filename Name of file.
 * @returns File extension.
 */
function getFileType(filename) {
    return filename.slice(filename.lastIndexOf(".") + 1);
}
