"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Record = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const view_1 = require("@codemirror/view");
const react_1 = require("@liqvid/keymap/react");
const codemirror_1 = require("@lqv/codemirror");
const extensions_1 = require("@lqv/codemirror/extensions");
const react_2 = require("react");
const extensions_2 = require("../extensions.cjs");
const store_1 = require("../store.cjs");
const Editor_1 = require("./Editor.cjs");
/** Recording editor. */
const Record = (props) => {
    const { captureKeys = {
        "Mod-Enter": "run",
        "Mod-K": "clear",
        "Mod-L": "clear",
    }, extensions = [], group = "default", passKeys = ["Mod-Alt-2", "Mod-Alt-3", "Mod-Alt-4"], ...attrs } = props;
    const store = (0, store_1.useBoothStore)();
    const lqvKeymap = (0, react_1.useKeymap)();
    const newExtensions = (0, react_2.useMemo)(() => (lqvKeymap ? [view_1.keymap.of((0, extensions_1.passThrough)(lqvKeymap, passKeys)), ...extensions] : extensions), [passKeys]);
    // attach recording extensions --- this has to be done this way because
    // the `shortcuts` Compartment will abort further handling of the sequence
    (0, react_2.useEffect)(() => {
        const state = store.getState();
        const { view } = state.groups[group].files.find((file) => file.filename === props.filename);
        view.dispatch({
            effects: extensions_2.recording.reconfigure([
                ...extensions_2.recording.get(view.state),
                [state.recorder.extension(captureKeys)],
            ]),
        });
        includeFilenameInRecording(state);
    }, []);
    return (0, jsx_runtime_1.jsx)(Editor_1.Editor, { content: props.content, extensions: newExtensions, ...attrs });
};
exports.Record = Record;
/* NOOOOOOOOOO */
const modifiedRecorder = Symbol();
function includeFilenameInRecording(state) {
    // only do this if we are recording in multiple files
    let recordingExtensions = 0;
    outer: {
        for (const group of Object.values(state.groups)) {
            for (const { view } of group.files) {
                const extnState = extensions_2.recording.get(view.state);
                if (extnState instanceof Array && extnState.length > 0) {
                    recordingExtensions++;
                    if (recordingExtensions > 1)
                        break outer;
                }
            }
        }
    }
    if (recordingExtensions < 2) {
        return;
    }
    // be idempotent
    if (state.recorder[modifiedRecorder]) {
        return;
    }
    // intercept beginRecording
    const beginRecording = state.recorder.beginRecording.bind(state.recorder);
    state.recorder.beginRecording = (...args) => {
        // have to call existing beginRecording() FIRST in order to
        // set state.recorder.duration, otherwise we get negative times!
        beginRecording(...args);
        state.recorder.capture(0, codemirror_1.selectCmd + state.getActiveFile().filename);
    };
    state.recorder[modifiedRecorder] = true;
}
