"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReplayMultiple = exports.Replay = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const view_1 = require("@codemirror/view");
const codemirror_1 = require("@lqv/codemirror");
const react_1 = require("@lqv/playback/react");
const react_2 = require("react");
const store_1 = require("../store.cjs");
const Editor_1 = require("./Editor.cjs");
/**
 * Editor to replay recorded coding.
 */
function Replay(props) {
    const store = (0, store_1.useBoothStore)();
    const playback = (0, react_1.useME)();
    const { extensions = [], handle, replay, scrollBehavior, shouldScroll, start = 0, ...attrs } = props;
    const __handle = (0, react_2.useCallback)((cmd, doc) => {
        if (cmd === "run") {
            // run command
            store.setState((state) => ({ run: state.run + 1 }));
        }
        else if (cmd === "clear") {
            // clear console
            store.setState(() => ({ messages: [] }));
        }
        // userspace handler
        if (props.handle) {
            props.handle(store, cmd, doc);
        }
    }, [handle]);
    const __extensions = (0, react_2.useMemo)(() => [
        view_1.ViewPlugin.define((0, codemirror_1.fakeSelection)((0, view_1.drawSelection)())),
        view_1.ViewPlugin.define((view) => {
            if (replay) {
                if (replay instanceof Promise) {
                    replay.then((data) => (0, codemirror_1.cmReplay)({
                        data,
                        handle: __handle,
                        playback,
                        scrollBehavior,
                        shouldScroll,
                        start,
                        view,
                    }));
                }
                else {
                    (0, codemirror_1.cmReplay)({
                        data: replay,
                        handle: __handle,
                        playback,
                        scrollBehavior,
                        shouldScroll,
                        start,
                        view,
                    });
                }
            }
            return {};
        }),
        ...extensions,
    ], [extensions, replay, start]);
    return (0, jsx_runtime_1.jsx)(Editor_1.Editor, { editable: false, extensions: __extensions, ...attrs });
}
exports.Replay = Replay;
/**
 * Replay coding to multiple editors.
 */
function ReplayMultiple(props) {
    const playback = (0, react_1.useME)();
    const store = (0, store_1.useBoothStore)();
    const { start = 0 } = props;
    /* Handle callback */
    const handle = (0, react_2.useCallback)((cmd, docs) => {
        // file selection change
        if (cmd.startsWith(codemirror_1.selectCmd)) {
            store.setState((state) => ({
                groups: {
                    ...state.groups,
                    [props.group]: {
                        ...state.groups[props.group],
                        activeFile: cmd.slice(codemirror_1.selectCmd.length),
                    },
                },
            }));
        }
        else if (cmd === "run") {
            // run command
            store.setState((state) => ({ run: state.run + 1 }));
        }
        else if (cmd === "clear") {
            // clear console
            store.setState(() => ({ messages: [] }));
        }
        // userspace handler
        if (props.handle) {
            props.handle(store, cmd, docs);
        }
    }, [props.handle]);
    (0, react_2.useEffect)(() => {
        const state = store.getState();
        const views = {};
        for (const file of state.groups[props.group].files) {
            views[file.filename] = file.view;
        }
        if (props.replay instanceof Promise) {
            props.replay.then((data) => (0, codemirror_1.cmReplayMultiple)({
                data,
                handle,
                playback,
                scrollBehavior: props.scrollBehavior,
                shouldScroll: props.shouldScroll,
                start,
                views,
            }));
        }
        else {
            (0, codemirror_1.cmReplayMultiple)({
                data: props.replay,
                handle,
                playback,
                scrollBehavior: props.scrollBehavior,
                shouldScroll: props.shouldScroll,
                start,
                views,
            });
        }
    }, []);
    return null;
}
exports.ReplayMultiple = ReplayMultiple;
