"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TabList = exports.Tab = exports.Run = exports.Reset = exports.Copy = exports.Clear = exports.Buttons = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("@liqvid/utils/react");
const classnames_1 = __importDefault(require("classnames"));
const react_2 = require("react");
const zustand_1 = require("zustand");
const store_1 = require("../store.cjs");
const utils_1 = require("../utils.cjs");
/** Div to hold buttons. */
function Buttons({ className, children }) {
    return (0, jsx_runtime_1.jsx)("div", { className: (0, classnames_1.default)("lqv-cb-buttons", className), children: children });
}
exports.Buttons = Buttons;
/** Button for clearing the output/console. */
function Clear({ className, children, ...attrs }) {
    const store = (0, store_1.useBoothStore)();
    const clear = (0, react_2.useCallback)(() => {
        store.setState({ messages: [] });
    }, []);
    const events = (0, react_2.useMemo)(() => (0, react_1.onClick)(clear), []);
    /* add keyboard shortcuts */
    (0, react_2.useEffect)(() => {
        store.setState((prev) => ({
            shortcuts: {
                ...prev.shortcuts,
                "Mod-L": {
                    key: "Mod-L",
                    run: () => {
                        clear();
                        return true;
                    },
                },
            },
        }));
    }, []);
    const label = "Clear";
    return ((0, jsx_runtime_1.jsx)("button", { "aria-label": label, className: (0, classnames_1.default)("lqv-cb-clear", className), title: label, ...events, ...attrs, children: children ?? label }));
}
exports.Clear = Clear;
/** Button for copying the contents of one group to another. */
function Copy({ children, className, from: fromGroup, to: toGroup, ...attrs }) {
    const store = (0, store_1.useBoothStore)();
    const copy = (0, react_2.useCallback)(() => {
        const { groups } = store.getState();
        const from = groups[fromGroup], to = groups[toGroup];
        if (!(from && to)) {
            console.error(`Could not copy from ${fromGroup} to ${toGroup}`);
            return;
        }
        for (const file of from.files) {
            const source = file.view;
            const target = to.files.find((_) => _.filename === file.filename).view;
            target.dispatch(target.state.update({
                changes: {
                    from: 0,
                    to: target.state.doc.length,
                    insert: source.state.doc,
                },
            }));
        }
    }, []);
    const events = (0, react_2.useMemo)(() => (0, react_1.onClick)(copy), []);
    return ((0, jsx_runtime_1.jsx)("button", { className: (0, classnames_1.default)("lqv-cb-copy", className), ...events, ...attrs, children: children ?? "Copy" }));
}
exports.Copy = Copy;
/** Button for resetting editor contents to initial state. */
function Reset({ className, children, ...attrs }) {
    const store = (0, store_1.useBoothStore)();
    const contents = (0, react_2.useRef)({});
    /* get contents */
    (0, react_2.useEffect)(() => {
        const state = store.getState();
        for (const key in state.groups) {
            contents.current[key] = {};
            for (const file of state.groups[key].files) {
                contents.current[key][file.filename] = file.view.state.doc.toString();
            }
        }
    }, []);
    /* reset */
    const reset = (0, react_2.useCallback)(() => {
        const state = store.getState();
        for (const groupName in contents.current) {
            for (const file of state.groups[groupName].files) {
                if (file.filename in contents.current[groupName]) {
                    file.view.dispatch(file.view.state.update({
                        changes: {
                            from: 0,
                            to: file.view.state.doc.length,
                            insert: contents.current[groupName][file.filename],
                        },
                    }));
                }
            }
        }
    }, []);
    const resetEvents = (0, react_2.useMemo)(() => (0, react_1.onClick)(reset), []);
    const label = "Reset";
    return ((0, jsx_runtime_1.jsx)("button", { className: (0, classnames_1.default)("lqv-cb-reset", className), title: label, ...resetEvents, ...attrs, children: children ?? "Reset" }));
}
exports.Reset = Reset;
/** Button for running the code. */
function Run({ className, children, ...attrs }) {
    const store = (0, store_1.useBoothStore)();
    // run callback
    const run = (0, react_2.useCallback)(() => {
        store.setState((prev) => ({ run: prev.run + 1 }));
    }, []);
    /* add keyboard shortcuts */
    (0, react_2.useEffect)(() => {
        store.setState((prev) => ({
            shortcuts: {
                ...prev.shortcuts,
                "Mod-Enter": {
                    key: "Mod-Enter",
                    run: () => {
                        run();
                        return true;
                    },
                },
            },
        }));
    }, []);
    // click events
    const events = (0, react_2.useMemo)(() => (0, react_1.onClick)(run), []);
    return ((0, jsx_runtime_1.jsx)("button", { className: (0, classnames_1.default)("lqv-cb-run", className), ...events, ...attrs, children: children ?? "Run" }));
}
exports.Run = Run;
/** Group selection tab. */
const Tab = (props) => {
    const { children, id, ...attrs } = props;
    const store = (0, store_1.useBoothStore)();
    const active = (0, zustand_1.useStore)(store, (state) => state.activeGroup === id);
    const events = (0, react_2.useMemo)(() => (0, react_1.onClick)(() => {
        store.setState({ activeGroup: id });
    }), []);
    return ((0, jsx_runtime_1.jsx)("button", { "aria-controls": utils_1.ids.editorGroup({ group: id }), "aria-selected": active, id: utils_1.ids.groupTab({ group: id }), role: "tab", ...events, ...attrs, children: props.children }));
};
exports.Tab = Tab;
/** Holds a list of {@link Tab}s. */
const TabList = (props) => {
    const { children, ...attrs } = props;
    return ((0, jsx_runtime_1.jsx)("div", { role: "tablist", ...attrs, children: children }));
};
exports.TabList = TabList;
