"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTMLConsole = exports.extensionFromFilename = exports.transform = exports.HTMLPreview = exports.HTMLRecord = exports.HTMLReplay = exports.HTMLDemo = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const lang_css_1 = require("@codemirror/lang-css");
const lang_html_1 = require("@codemirror/lang-html");
const lang_javascript_1 = require("@codemirror/lang-javascript");
const recording_1 = require("@lqv/codemirror/recording");
const react_1 = require("react");
const zustand_1 = require("zustand");
const __1 = require("../index.cjs");
const buttons_1 = require("../components/buttons.cjs");
const Console_1 = require("../components/Console.cjs");
const Editor_1 = require("../components/Editor.cjs");
const EditorGroup_1 = require("../components/EditorGroup.cjs");
const EditorPanel_1 = require("../components/EditorPanel.cjs");
const FileTabs_1 = require("../components/FileTabs.cjs");
const Record_1 = require("../components/Record.cjs");
const Replay_1 = require("../components/Replay.cjs");
const Resize_1 = require("../components/Resize.cjs");
const extensions_1 = require("../extensions.cjs");
const store_1 = require("../store.cjs");
/** HTML demo */
function HTMLDemo(props) {
    const { extensions = [] } = props;
    return ((0, jsx_runtime_1.jsxs)(__1.CodeBooth, { children: [(0, jsx_runtime_1.jsx)(FileTabs_1.FileTabs, {}), Object.keys(props.files).map((filename) => ((0, jsx_runtime_1.jsx)(EditorPanel_1.EditorPanel, { filename: filename, children: (0, jsx_runtime_1.jsx)(Editor_1.Editor, { content: props.files[filename], extensions: [extensions_1.basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))), (0, jsx_runtime_1.jsx)(Resize_1.Resize, {}), (0, jsx_runtime_1.jsx)(Resize_1.Resize, { dir: "ns" }), (0, jsx_runtime_1.jsx)(HTMLPreview, {}), (0, jsx_runtime_1.jsx)(Console_1.Console, {}), (0, jsx_runtime_1.jsxs)(buttons_1.Buttons, { children: [(0, jsx_runtime_1.jsx)(buttons_1.Reset, {}), (0, jsx_runtime_1.jsx)(buttons_1.Run, {}), (0, jsx_runtime_1.jsx)(buttons_1.Clear, {})] }), props.children] }));
}
exports.HTMLDemo = HTMLDemo;
/** Interactive HTML replay. */
const HTMLReplay = (props) => {
    const { extensions = [] } = props;
    return ((0, jsx_runtime_1.jsxs)(__1.CodeBooth, { children: [(0, jsx_runtime_1.jsx)(FileTabs_1.FileTabs, {}), (0, jsx_runtime_1.jsxs)(EditorGroup_1.EditorGroup, { id: "replay", children: [Object.keys(props.files).map((filename) => ((0, jsx_runtime_1.jsx)(EditorPanel_1.EditorPanel, { filename: filename, children: (0, jsx_runtime_1.jsx)(Replay_1.Replay, { content: props.files[filename], extensions: [extensions_1.basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))), (0, jsx_runtime_1.jsx)(Replay_1.ReplayMultiple, { replay: props.replay, start: props.start })] }), (0, jsx_runtime_1.jsx)(EditorGroup_1.EditorGroup, { id: "playground", children: Object.keys(props.files).map((filename) => ((0, jsx_runtime_1.jsx)(EditorPanel_1.EditorPanel, { filename: filename, children: (0, jsx_runtime_1.jsx)(Editor_1.Editor, { extensions: [extensions_1.basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))) }), (0, jsx_runtime_1.jsx)(Resize_1.Resize, {}), (0, jsx_runtime_1.jsx)(Resize_1.Resize, { dir: "ns" }), (0, jsx_runtime_1.jsx)(HTMLPreview, {}), (0, jsx_runtime_1.jsx)(Console_1.Console, {}), (0, jsx_runtime_1.jsxs)(buttons_1.Buttons, { children: [(0, jsx_runtime_1.jsxs)(buttons_1.TabList, { children: [(0, jsx_runtime_1.jsx)(buttons_1.Tab, { id: "replay", children: "Replay" }), (0, jsx_runtime_1.jsx)(buttons_1.Tab, { id: "playground", children: "Playground" })] }), (0, jsx_runtime_1.jsx)(buttons_1.Copy, { from: "replay", to: "playground" }), (0, jsx_runtime_1.jsx)(buttons_1.Run, {}), (0, jsx_runtime_1.jsx)(buttons_1.Clear, {})] }), props.children] }));
};
exports.HTMLReplay = HTMLReplay;
/** Record HTML demos. */
const HTMLRecord = (props) => {
    const { extensions = [], recorder = recording_1.CodeRecording.recorder } = props;
    return ((0, jsx_runtime_1.jsxs)(__1.CodeBooth, { recorder: recorder, children: [(0, jsx_runtime_1.jsx)(FileTabs_1.FileTabs, {}), Object.keys(props.files).map((filename) => ((0, jsx_runtime_1.jsx)(EditorPanel_1.EditorPanel, { filename: filename, children: (0, jsx_runtime_1.jsx)(Record_1.Record, { content: props.files[filename], extensions: [extensions_1.basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))), (0, jsx_runtime_1.jsx)(Resize_1.Resize, {}), (0, jsx_runtime_1.jsx)(Resize_1.Resize, { dir: "ns" }), (0, jsx_runtime_1.jsx)(HTMLPreview, {}), (0, jsx_runtime_1.jsx)(Console_1.Console, {}), (0, jsx_runtime_1.jsxs)(buttons_1.Buttons, { children: [(0, jsx_runtime_1.jsx)(buttons_1.Run, {}), (0, jsx_runtime_1.jsx)(buttons_1.Clear, {})] }), props.children] }));
};
exports.HTMLRecord = HTMLRecord;
/** Preview of HTML document. */
function HTMLPreview() {
    const store = (0, store_1.useBoothStore)();
    /** <iframe> containing preview document */
    const iframe = (0, react_1.useRef)();
    /* rendering */
    (0, react_1.useEffect)(() => {
        /* re-render */
        function render() {
            const files = getFileContents(store.getState());
            if (!("index.html" in files)) {
                return;
            }
            iframe.current.srcdoc = transform(files["index.html"], files);
        }
        // initial render
        render();
        /* unsubscriptions */
        const unsubs = [];
        /* messaging */
        function update(msg) {
            if (msg.data.type === "console.log") {
                store.setState((prev) => ({
                    ...prev,
                    messages: prev.messages.concat((0, jsx_runtime_1.jsx)("pre", { children: msg.data.content.map((item) => formatLog(item)) }, Math.random())),
                }));
            }
            else if (msg.data.type === "console.clear") {
                store.setState((prev) => ({
                    ...prev,
                    messages: [],
                }));
            }
        }
        window.addEventListener("message", update);
        unsubs.push(() => {
            window.removeEventListener("message", update);
        });
        // subscribe to run event
        unsubs.push(store.subscribe((state) => state.run, () => {
            render();
            // iframe.current.contentWindow.postMessage({
            //   type: "update-css",
            //   filename: file.filename,
            //   content
            // }, "*");
        }));
        // unsubscribe
        return () => {
            for (const unsub of unsubs) {
                unsub();
            }
        };
    }, []);
    return (0, jsx_runtime_1.jsx)("iframe", { className: "lqv-preview", ref: iframe, sandbox: "allow-popups allow-scripts" });
}
exports.HTMLPreview = HTMLPreview;
/**
 * Inlines scripts and stylesheets in HTML code.
 * @param html HTML code to transform
 * @param files Map of file
 * @returns Transformed HTML code.
 */
function transform(html, files) {
    // transform <script>s
    html = html.replace(/<script\s*src=(['"])(?:\.\/)?([^\1]+?\.js)\1\s*><\/script>/gi, (match, q, src) => {
        if (src in files) {
            return "<script>" + files[src] + "</script>";
        }
        return match;
    });
    // transform <link>s
    html = html.replace(/<link([^>]+?)>/gi, (match, attrs) => {
        const $_ = attrs.match(/href=(['"])(?:\.\/)?([^\1]+?\.css)\1/);
        if ($_) {
            const href = $_[2];
            if (href in files) {
                return `<style data-filename="${href}">` + files[href] + "</style>";
            }
            return match;
        }
        return match;
    });
    // magic scripts
    html = html.replace("<head>", "<head>" + magicScripts);
    // return
    return html;
}
exports.transform = transform;
/** Iframe client code for development magic */
const magicScripts = String.raw `<script>
/* update CSS without reloading */
window.addEventListener("message", ({data}) => {
  if (data.type === "update-css") {
    document.querySelector("style[data-filename='" + data.filename + "']").textContent = data.content;
  }
});

/* intercept console.log */
{
  function formatArgs(args) {
    if (args instanceof Array) {
      return args.filter(item => !(item instanceof Node)).map(formatArgs);
    }
    return args;
  }
  const log = console.log;
  console.log = function(...args) {
    try {
      window.parent.postMessage({
        type: "console.log",
        content: formatArgs(args)
      }, "*");
    } catch (e) {
      
    }
    log(...args);
  }

  const clear = console.clear;
  console.clear = function() {
    window.parent.postMessage({
      type: "console.clear"
    }, "*");
    clear();
  }
}
</script>`;
/**
 * Get CodeMirror {@link Extension} appropriate to filename extension
 * @param filename Name of file.
 * @returns Either `css()`, `html()`, or `javascript()` from `@codemirror/lang-*`.
 */
function extensionFromFilename(filename) {
    switch (true) {
        case filename.endsWith(".css"):
            return (0, lang_css_1.css)();
        case filename.endsWith(".html"):
            return (0, lang_html_1.html)();
        case filename.endsWith(".js"):
            return (0, lang_javascript_1.javascript)();
    }
}
exports.extensionFromFilename = extensionFromFilename;
/**
 * Get record of filenames to file contents.
 * @param state Booth state.
 */
function getFileContents(state) {
    const ret = {};
    if (!state.groups[state.activeGroup])
        return ret;
    const { files } = state.groups[state.activeGroup];
    for (const file of files) {
        ret[file.filename] = file.view.state.doc.toString();
    }
    return ret;
}
/**
 * Format console logs.
 * @param o Message or object to log.
 * @param formatString Whether to format the log entry.
 */
function formatLog(o, formatString = false) {
    if (o instanceof Array) {
        return ((0, jsx_runtime_1.jsxs)("span", { className: "array", children: ["Array [", " ", (0, jsx_runtime_1.jsx)("ol", { children: o.map((entry, i) => ((0, jsx_runtime_1.jsx)("li", { children: formatLog(entry, true) }, i))) }), " ", "]"] }, key()));
    }
    else if (typeof o === "object") {
        return ((0, jsx_runtime_1.jsxs)("span", { className: "object", children: ["Object {", " ", (0, jsx_runtime_1.jsx)("ol", { children: Object.keys(o).map((k, i) => ((0, jsx_runtime_1.jsxs)("li", { children: [k, ": ", formatLog(o[k], true)] }, i))) }), " ", "}"] }, key()));
    }
    else if (typeof o === "number") {
        return ((0, jsx_runtime_1.jsx)("span", { className: "number", children: o }, key()));
    }
    else if (typeof o === "string") {
        if (formatString) {
            return ((0, jsx_runtime_1.jsxs)("span", { className: "string", children: ["\"", o, "\""] }, key()));
        }
        else {
            return (0, jsx_runtime_1.jsx)("span", { children: o }, key());
        }
    }
    else if (typeof o === "undefined") {
        return ((0, jsx_runtime_1.jsx)("span", { className: "undefined", children: "undefined" }, key()));
    }
    return o.toString();
}
/** Get unique React key. */
function key() {
    return Math.random();
}
/**
 * Component for displaying console logs.
 */
const HTMLConsole = () => {
    const store = (0, store_1.useBoothStore)();
    const messages = (0, zustand_1.useStore)(store, (state) => state.messages);
    return ((0, jsx_runtime_1.jsx)("section", { className: "lqv-console", children: (0, jsx_runtime_1.jsx)("output", { children: messages }) }));
};
exports.HTMLConsole = HTMLConsole;
