"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PythonRun = exports.PythonReplay = exports.PythonRecord = exports.PythonDemo = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const lang_python_1 = require("@codemirror/lang-python");
const recording_1 = require("@lqv/codemirror/recording");
const react_1 = require("react");
const __1 = require("../index.cjs");
const buttons_1 = require("../components/buttons.cjs");
const Console_1 = require("../components/Console.cjs");
const Editor_1 = require("../components/Editor.cjs");
const EditorGroup_1 = require("../components/EditorGroup.cjs");
const Record_1 = require("../components/Record.cjs");
const Replay_1 = require("../components/Replay.cjs");
const Resize_1 = require("../components/Resize.cjs");
const extensions_1 = require("../extensions.cjs");
const skulpt_1 = require("../interpreters/skulpt.cjs");
const store_1 = require("../store.cjs");
const interpreter = new skulpt_1.PythonInterpreter();
/** Python demo */
const PythonDemo = (props) => {
    const { extensions = [] } = props;
    return ((0, jsx_runtime_1.jsxs)(__1.CodeBooth, { children: [(0, jsx_runtime_1.jsx)(Editor_1.Editor, { content: props.content, extensions: [extensions_1.basicSetup, (0, lang_python_1.python)(), ...extensions], filename: "untitled.py" }), (0, jsx_runtime_1.jsx)(Resize_1.Resize, {}), (0, jsx_runtime_1.jsx)(exports.PythonRun, {}), (0, jsx_runtime_1.jsx)(Console_1.Console, {}), (0, jsx_runtime_1.jsxs)(buttons_1.Buttons, { children: [(0, jsx_runtime_1.jsx)(buttons_1.Reset, {}), (0, jsx_runtime_1.jsx)(buttons_1.Run, {}), (0, jsx_runtime_1.jsx)(buttons_1.Clear, {})] })] }));
};
exports.PythonDemo = PythonDemo;
/** Record Python coding. */
const PythonRecord = (props) => {
    const { extensions = [] } = props;
    return ((0, jsx_runtime_1.jsxs)(__1.CodeBooth, { recorder: recording_1.CodeRecording.recorder, children: [(0, jsx_runtime_1.jsx)(Record_1.Record, { content: props.content, extensions: [(0, lang_python_1.python)(), extensions_1.basicSetup, ...extensions], filename: "untitled.py" }), (0, jsx_runtime_1.jsx)(Resize_1.Resize, {}), (0, jsx_runtime_1.jsx)(exports.PythonRun, {}), (0, jsx_runtime_1.jsx)(Console_1.Console, {}), (0, jsx_runtime_1.jsxs)(buttons_1.Buttons, { children: [(0, jsx_runtime_1.jsx)(buttons_1.Run, {}), (0, jsx_runtime_1.jsx)(buttons_1.Clear, {})] })] }));
};
exports.PythonRecord = PythonRecord;
/** Interactive Python replay. */
const PythonReplay = (props) => {
    const { extensions = [] } = props;
    return ((0, jsx_runtime_1.jsxs)(__1.CodeBooth, { children: [(0, jsx_runtime_1.jsx)(EditorGroup_1.EditorGroup, { id: "replay", children: (0, jsx_runtime_1.jsx)(Replay_1.Replay, { content: props.content, extensions: [extensions_1.basicSetup, (0, lang_python_1.python)(), ...extensions], filename: "untitled.py", replay: props.replay, start: props.start }) }), (0, jsx_runtime_1.jsx)(EditorGroup_1.EditorGroup, { id: "playground", children: (0, jsx_runtime_1.jsx)(Editor_1.Editor, { content: props.content, extensions: [extensions_1.basicSetup, (0, lang_python_1.python)(), ...extensions], filename: "untitled.py" }) }), (0, jsx_runtime_1.jsx)(Resize_1.Resize, {}), (0, jsx_runtime_1.jsx)(exports.PythonRun, {}), (0, jsx_runtime_1.jsx)(Console_1.Console, {}), (0, jsx_runtime_1.jsxs)(buttons_1.Buttons, { children: [(0, jsx_runtime_1.jsxs)(buttons_1.TabList, { children: [(0, jsx_runtime_1.jsx)(buttons_1.Tab, { id: "replay", children: "Replay" }), (0, jsx_runtime_1.jsx)(buttons_1.Tab, { id: "playground", children: "Playground" })] }), (0, jsx_runtime_1.jsx)(buttons_1.Copy, { from: "replay", to: "playground" }), (0, jsx_runtime_1.jsx)(buttons_1.Run, {}), (0, jsx_runtime_1.jsx)(buttons_1.Clear, {})] }), props.children] }));
};
exports.PythonReplay = PythonReplay;
/** Run Python code. */
const PythonRun = () => {
    const store = (0, store_1.useBoothStore)();
    (0, react_1.useEffect)(() => {
        return store.subscribe((state) => state.run, () => {
            const state = store.getState();
            const view = state.getActiveView();
            const code = view.state.doc.toString();
            let output = [];
            try {
                output = interpreter.runSync(code).map((log) => (0, jsx_runtime_1.jsx)("pre", { children: log }, Math.random()));
            }
            catch (e) {
                const msg = `Error (line ${e.traceback[0].lineno}): ${e.args.v[0].v}`;
                output = [
                    (0, jsx_runtime_1.jsx)("pre", { className: "error", children: msg }, `${msg}-${Math.random()}`),
                ];
            }
            store.setState((prev) => ({
                messages: [...prev.messages, ...output],
            }));
        });
    }, []);
    return null;
};
exports.PythonRun = PythonRun;
