import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { css } from "@codemirror/lang-css";
import { html } from "@codemirror/lang-html";
import { javascript } from "@codemirror/lang-javascript";
import { CodeRecording } from "@lqv/codemirror/recording";
import { useEffect, useRef } from "react";
import { useStore } from "zustand";
import { CodeBooth } from "../index.mjs";
import { Buttons, Clear, Copy, Reset, Run, Tab, TabList } from "../components/buttons.mjs";
import { Console } from "../components/Console.mjs";
import { Editor } from "../components/Editor.mjs";
import { EditorGroup } from "../components/EditorGroup.mjs";
import { EditorPanel } from "../components/EditorPanel.mjs";
import { FileTabs } from "../components/FileTabs.mjs";
import { Record } from "../components/Record.mjs";
import { Replay, ReplayMultiple } from "../components/Replay.mjs";
import { Resize } from "../components/Resize.mjs";
import { basicSetup } from "../extensions.mjs";
import { useBoothStore } from "../store.mjs";
/** HTML demo */
export function HTMLDemo(props) {
    const { extensions = [] } = props;
    return (_jsxs(CodeBooth, { children: [_jsx(FileTabs, {}), Object.keys(props.files).map((filename) => (_jsx(EditorPanel, { filename: filename, children: _jsx(Editor, { content: props.files[filename], extensions: [basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))), _jsx(Resize, {}), _jsx(Resize, { dir: "ns" }), _jsx(HTMLPreview, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsx(Reset, {}), _jsx(Run, {}), _jsx(Clear, {})] }), props.children] }));
}
/** Interactive HTML replay. */
export const HTMLReplay = (props) => {
    const { extensions = [] } = props;
    return (_jsxs(CodeBooth, { children: [_jsx(FileTabs, {}), _jsxs(EditorGroup, { id: "replay", children: [Object.keys(props.files).map((filename) => (_jsx(EditorPanel, { filename: filename, children: _jsx(Replay, { content: props.files[filename], extensions: [basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))), _jsx(ReplayMultiple, { replay: props.replay, start: props.start })] }), _jsx(EditorGroup, { id: "playground", children: Object.keys(props.files).map((filename) => (_jsx(EditorPanel, { filename: filename, children: _jsx(Editor, { extensions: [basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))) }), _jsx(Resize, {}), _jsx(Resize, { dir: "ns" }), _jsx(HTMLPreview, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsxs(TabList, { children: [_jsx(Tab, { id: "replay", children: "Replay" }), _jsx(Tab, { id: "playground", children: "Playground" })] }), _jsx(Copy, { from: "replay", to: "playground" }), _jsx(Run, {}), _jsx(Clear, {})] }), props.children] }));
};
/** Record HTML demos. */
export const HTMLRecord = (props) => {
    const { extensions = [], recorder = CodeRecording.recorder } = props;
    return (_jsxs(CodeBooth, { recorder: recorder, children: [_jsx(FileTabs, {}), Object.keys(props.files).map((filename) => (_jsx(EditorPanel, { filename: filename, children: _jsx(Record, { content: props.files[filename], extensions: [basicSetup, extensionFromFilename(filename), ...extensions] }) }, filename))), _jsx(Resize, {}), _jsx(Resize, { dir: "ns" }), _jsx(HTMLPreview, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsx(Run, {}), _jsx(Clear, {})] }), props.children] }));
};
/** Preview of HTML document. */
export function HTMLPreview() {
    const store = useBoothStore();
    /** <iframe> containing preview document */
    const iframe = useRef();
    /* rendering */
    useEffect(() => {
        /* re-render */
        function render() {
            const files = getFileContents(store.getState());
            if (!("index.html" in files)) {
                return;
            }
            iframe.current.srcdoc = transform(files["index.html"], files);
        }
        // initial render
        render();
        /* unsubscriptions */
        const unsubs = [];
        /* messaging */
        function update(msg) {
            if (msg.data.type === "console.log") {
                store.setState((prev) => ({
                    ...prev,
                    messages: prev.messages.concat(_jsx("pre", { children: msg.data.content.map((item) => formatLog(item)) }, Math.random())),
                }));
            }
            else if (msg.data.type === "console.clear") {
                store.setState((prev) => ({
                    ...prev,
                    messages: [],
                }));
            }
        }
        window.addEventListener("message", update);
        unsubs.push(() => {
            window.removeEventListener("message", update);
        });
        // subscribe to run event
        unsubs.push(store.subscribe((state) => state.run, () => {
            render();
            // iframe.current.contentWindow.postMessage({
            //   type: "update-css",
            //   filename: file.filename,
            //   content
            // }, "*");
        }));
        // unsubscribe
        return () => {
            for (const unsub of unsubs) {
                unsub();
            }
        };
    }, []);
    return _jsx("iframe", { className: "lqv-preview", ref: iframe, sandbox: "allow-popups allow-scripts" });
}
/**
 * Inlines scripts and stylesheets in HTML code.
 * @param html HTML code to transform
 * @param files Map of file
 * @returns Transformed HTML code.
 */
export function transform(html, files) {
    // transform <script>s
    html = html.replace(/<script\s*src=(['"])(?:\.\/)?([^\1]+?\.js)\1\s*><\/script>/gi, (match, q, src) => {
        if (src in files) {
            return "<script>" + files[src] + "</script>";
        }
        return match;
    });
    // transform <link>s
    html = html.replace(/<link([^>]+?)>/gi, (match, attrs) => {
        const $_ = attrs.match(/href=(['"])(?:\.\/)?([^\1]+?\.css)\1/);
        if ($_) {
            const href = $_[2];
            if (href in files) {
                return `<style data-filename="${href}">` + files[href] + "</style>";
            }
            return match;
        }
        return match;
    });
    // magic scripts
    html = html.replace("<head>", "<head>" + magicScripts);
    // return
    return html;
}
/** Iframe client code for development magic */
const magicScripts = String.raw `<script>
/* update CSS without reloading */
window.addEventListener("message", ({data}) => {
  if (data.type === "update-css") {
    document.querySelector("style[data-filename='" + data.filename + "']").textContent = data.content;
  }
});

/* intercept console.log */
{
  function formatArgs(args) {
    if (args instanceof Array) {
      return args.filter(item => !(item instanceof Node)).map(formatArgs);
    }
    return args;
  }
  const log = console.log;
  console.log = function(...args) {
    try {
      window.parent.postMessage({
        type: "console.log",
        content: formatArgs(args)
      }, "*");
    } catch (e) {
      
    }
    log(...args);
  }

  const clear = console.clear;
  console.clear = function() {
    window.parent.postMessage({
      type: "console.clear"
    }, "*");
    clear();
  }
}
</script>`;
/**
 * Get CodeMirror {@link Extension} appropriate to filename extension
 * @param filename Name of file.
 * @returns Either `css()`, `html()`, or `javascript()` from `@codemirror/lang-*`.
 */
export function extensionFromFilename(filename) {
    switch (true) {
        case filename.endsWith(".css"):
            return css();
        case filename.endsWith(".html"):
            return html();
        case filename.endsWith(".js"):
            return javascript();
    }
}
/**
 * Get record of filenames to file contents.
 * @param state Booth state.
 */
function getFileContents(state) {
    const ret = {};
    if (!state.groups[state.activeGroup])
        return ret;
    const { files } = state.groups[state.activeGroup];
    for (const file of files) {
        ret[file.filename] = file.view.state.doc.toString();
    }
    return ret;
}
/**
 * Format console logs.
 * @param o Message or object to log.
 * @param formatString Whether to format the log entry.
 */
function formatLog(o, formatString = false) {
    if (o instanceof Array) {
        return (_jsxs("span", { className: "array", children: ["Array [", " ", _jsx("ol", { children: o.map((entry, i) => (_jsx("li", { children: formatLog(entry, true) }, i))) }), " ", "]"] }, key()));
    }
    else if (typeof o === "object") {
        return (_jsxs("span", { className: "object", children: ["Object {", " ", _jsx("ol", { children: Object.keys(o).map((k, i) => (_jsxs("li", { children: [k, ": ", formatLog(o[k], true)] }, i))) }), " ", "}"] }, key()));
    }
    else if (typeof o === "number") {
        return (_jsx("span", { className: "number", children: o }, key()));
    }
    else if (typeof o === "string") {
        if (formatString) {
            return (_jsxs("span", { className: "string", children: ["\"", o, "\""] }, key()));
        }
        else {
            return _jsx("span", { children: o }, key());
        }
    }
    else if (typeof o === "undefined") {
        return (_jsx("span", { className: "undefined", children: "undefined" }, key()));
    }
    return o.toString();
}
/** Get unique React key. */
function key() {
    return Math.random();
}
/**
 * Component for displaying console logs.
 */
export const HTMLConsole = () => {
    const store = useBoothStore();
    const messages = useStore(store, (state) => state.messages);
    return (_jsx("section", { className: "lqv-console", children: _jsx("output", { children: messages }) }));
};
