import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { python } from "@codemirror/lang-python";
import { CodeRecording } from "@lqv/codemirror/recording";
import { useEffect } from "react";
import { CodeBooth } from "../index.mjs";
import { Buttons, Clear, Copy, Reset, Run, Tab, TabList } from "../components/buttons.mjs";
import { Console } from "../components/Console.mjs";
import { Editor } from "../components/Editor.mjs";
import { EditorGroup } from "../components/EditorGroup.mjs";
import { Record } from "../components/Record.mjs";
import { Replay } from "../components/Replay.mjs";
import { Resize } from "../components/Resize.mjs";
import { basicSetup } from "../extensions.mjs";
import { PythonInterpreter } from "../interpreters/skulpt.mjs";
import { useBoothStore } from "../store.mjs";
const interpreter = new PythonInterpreter();
/** Python demo */
export const PythonDemo = (props) => {
    const { extensions = [] } = props;
    return (_jsxs(CodeBooth, { children: [_jsx(Editor, { content: props.content, extensions: [basicSetup, python(), ...extensions], filename: "untitled.py" }), _jsx(Resize, {}), _jsx(PythonRun, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsx(Reset, {}), _jsx(Run, {}), _jsx(Clear, {})] })] }));
};
/** Record Python coding. */
export const PythonRecord = (props) => {
    const { extensions = [] } = props;
    return (_jsxs(CodeBooth, { recorder: CodeRecording.recorder, children: [_jsx(Record, { content: props.content, extensions: [python(), basicSetup, ...extensions], filename: "untitled.py" }), _jsx(Resize, {}), _jsx(PythonRun, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsx(Run, {}), _jsx(Clear, {})] })] }));
};
/** Interactive Python replay. */
export const PythonReplay = (props) => {
    const { extensions = [] } = props;
    return (_jsxs(CodeBooth, { children: [_jsx(EditorGroup, { id: "replay", children: _jsx(Replay, { content: props.content, extensions: [basicSetup, python(), ...extensions], filename: "untitled.py", replay: props.replay, start: props.start }) }), _jsx(EditorGroup, { id: "playground", children: _jsx(Editor, { content: props.content, extensions: [basicSetup, python(), ...extensions], filename: "untitled.py" }) }), _jsx(Resize, {}), _jsx(PythonRun, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsxs(TabList, { children: [_jsx(Tab, { id: "replay", children: "Replay" }), _jsx(Tab, { id: "playground", children: "Playground" })] }), _jsx(Copy, { from: "replay", to: "playground" }), _jsx(Run, {}), _jsx(Clear, {})] }), props.children] }));
};
/** Run Python code. */
export const PythonRun = () => {
    const store = useBoothStore();
    useEffect(() => {
        return store.subscribe((state) => state.run, () => {
            const state = store.getState();
            const view = state.getActiveView();
            const code = view.state.doc.toString();
            let output = [];
            try {
                output = interpreter.runSync(code).map((log) => _jsx("pre", { children: log }, Math.random()));
            }
            catch (e) {
                const msg = `Error (line ${e.traceback[0].lineno}): ${e.args.v[0].v}`;
                output = [
                    _jsx("pre", { className: "error", children: msg }, `${msg}-${Math.random()}`),
                ];
            }
            store.setState((prev) => ({
                messages: [...prev.messages, ...output],
            }));
        });
    }, []);
    return null;
};
