/// <reference types="react" />
import type { EditorView, KeyBinding } from "@codemirror/view";
import type { CodeRecorder } from "@lqv/codemirror/recording";
type RecordType<T> = T extends Record<string, infer K> ? K : never;
type ArrayType<T> = T extends (infer K)[] ? K : never;
/** CodeBooth store state. */
export interface State {
    /**
     * Name of active editor group.
     */
    activeGroup: string;
    /** Class names to attach to root. */
    classNames: string[];
    /** Group of files/editors. */
    groups: Record<string, {
        /** Name of active file. */
        activeFile: string;
        /** Files contained in this editor group. */
        files: {
            /** Whether the buffer can be edited by the viewer. */
            editable: boolean;
            /** File name. */
            filename: string;
            /** Reference to CodeMirror {@link EditorView} */
            view: EditorView;
        }[];
    }>;
    /** Console logs. */
    messages: React.ReactNode[];
    /** Code recorder. */
    recorder?: CodeRecorder;
    /** Used to broadcast run events. */
    run: number;
    /** Keyboard shortcuts. */
    shortcuts: Record<string, KeyBinding>;
    /** Get the active file. */
    getActiveFile(): ArrayType<RecordType<State["groups"]>["files"]>;
    /** Get the active view. */
    getActiveView(): EditorView;
}
export declare const makeStore: (state?: Partial<State>) => Omit<import("zustand").StoreApi<State>, "subscribe"> & {
    subscribe: {
        (listener: (selectedState: State, previousSelectedState: State) => void): () => void;
        <U>(selector: (state: State) => U, listener: (selectedState: U, previousSelectedState: U) => void, options?: {
            equalityFn?: (a: U, b: U) => boolean;
            fireImmediately?: boolean;
        }): () => void;
    };
};
export type Store = ReturnType<typeof makeStore>;
export declare const BoothStore: import("react").Context<Omit<import("zustand").StoreApi<State>, "subscribe"> & {
    subscribe: {
        (listener: (selectedState: State, previousSelectedState: State) => void): () => void;
        <U>(selector: (state: State) => U, listener: (selectedState: U, previousSelectedState: U) => void, options?: {
            equalityFn?: (a: U, b: U) => boolean;
            fireImmediately?: boolean;
        }): () => void;
    };
}>;
/** Get a reference to the Zustand store for this CodeBooth. See {@link State} for store shape. */
export declare function useBoothStore(): Store;
export {};
